/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
/*
 * Image.java
 * Last modified by: $Author: amarkhel $
 * $Revision: 13476 $	$Date: 2009-04-09 18:58:20 +0300 (Thu, 09 Apr 2009) $
 */
package org.richfaces.realworld.domain;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.richfaces.realworld.service.ActionTools;

@NamedQueries({
    @NamedQuery(
            name = "album-unvisited",
            query = "from Image i where i.album=:album and i.uploaded > :date"
    ),
    @NamedQuery(
            name = "shelf-unvisited",
            query = "from Image i where i.album.shelf = :shelf and i.uploaded > :date"
    ),
    @NamedQuery(
            name = "tag-byName",
            query = "from MetaTag m where m.tag =:tag"
    ), 
    @NamedQuery(
            name = "tag-popular",
            query = "select new MetaTag(m.id, m.tag) from MetaTag m join m.images img group by m.id, m.tag order by count(img) desc"
    ), 
    @NamedQuery(
            name = "user-shelves",
            query = "select distinct s from Shelf s where (s.shared = true and s.preDefined = true) or s.owner = :user order by s.name"
    ), 
    @NamedQuery(
            name = "tag-suggest",
            query = "select m from MetaTag m where lower(m.tag) like :tag"
    )
        }
)
/**
 * Class for representing Image Entity
 *  EJB3 Entity Bean
 *
 * @author Andrey Markhel
 */

@Entity
@Name("image")
@Table(name = "Images")
@Scope(ScopeType.CONVERSATION)
@AutoCreate
public class Image implements Serializable {

    private static final long serialVersionUID = -7042878411608396483L;

    @Id
    @GeneratedValue
    @Column(name = "IMAGE_ID")
    private Long id = null;

    @Column(length = 255, nullable = false)
    @NotNull
    @NotEmpty
    @Length(min = 3)
    private String name;
    
    @Transient
    private boolean covering;

    @Column(length = 1024, nullable = false)
    @NotNull
    @NotEmpty
    @Length(min = 3)
    private String path;

    @ManyToMany(cascade = CascadeType.ALL)
    @JoinTable(name = "IMAGES_METATAGS",
            joinColumns =
            @JoinColumn(name = "CI_IMAGE_ID",
                    referencedColumnName = "IMAGE_ID"),
            inverseJoinColumns =
            @JoinColumn(name = "CI_METATAG_ID",
                    referencedColumnName = "METATAG_ID"))
    protected List<MetaTag> imageTags = new ArrayList<MetaTag>();

    @Transient
    private boolean visited;
    /*
     * Comma separated tag value
     * */
    @Transient
    private String meta = "";

    @Column(length = 255)
    private String cameraModel;

    private int height;

    private double size;

    private int width;

    @Temporal(TemporalType.TIMESTAMP)
    private Date uploaded;

    @Column(length = 1024)
    @NotNull
    @NotEmpty
    @Length(min = 3)
    private String description;

    @Temporal(TemporalType.TIMESTAMP)
    private Date created;

    @OneToMany(mappedBy = "image", cascade = {CascadeType.ALL}, fetch = FetchType.LAZY)
    @org.hibernate.annotations.LazyCollection(org.hibernate.annotations.LazyCollectionOption.EXTRA)
    @org.hibernate.annotations.OrderBy(clause = "date asc")
    private List<Comment> comments = new ArrayList<Comment>();

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "ALBUM_ID", referencedColumnName = "ALBUM_ID")
    private Album album;

    @NotNull
    private boolean allowComments;

    // ********************** Accessor Methods ********************** //

    /**
     * Getter for property id
     *
     * @return id of image
     */
    public Long getId() {
        return id;
    }

    /**
     * Getter for property name
     *
     * @return name of image
     */
    public String getName() {
        return name;
    }

    /**
     * Setter for property name
     *
     * @param name - name of image
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Getter for property description
     *
     * @return description of image
     */
    public String getDescription() {
        return description;
    }

    /**
     * Setter for property description
     *
     * @param description - description of image
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * Getter for property path.
     * Represent file-system structure, relative at uploadRoot dir(determined at startup, by default is system temp dir)
     * Usually is user.GetLogin() + SLASH + image.getAlbum().getId() + SLASH + fileName,
     * for example "amarkhel/15/coolPicture.jpg"
     *
     * @return relative path of image
     */
    public String getPath() {
        return path;
    }

    /**
     * Setter for property path
     *
     * @param path - relative path to image
     */
    public void setPath(String path) {
        this.path = path;
    }

    /**
     * Getter for property created
     *
     * @return date of creation of this image
     */
    public Date getCreated() {
        return created;
    }

    /**
     * Setter for property created
     *
     * @param created - date of creation
     */
    public void setCreated(Date created) {
        this.created = created;
    }

    /**
     * Getter for property album
     *
     * @return containing album
     */
    public Album getAlbum() {
        return album;
    }

    /**
     * Setter for property album
     *
     * @param album - album, that will contain this image
     */
    public void setAlbum(Album album) {
        this.album = album;
    }

    /**
     * Getter for property comments
     *
     * @return collection of comments, belongs to that image
     */
    public List<Comment> getComments() {
        return comments;
    }

    /**
     * Getter for property imageTags
     *
     * @return collection of MetaTags, associated to that image
     */
    public List<MetaTag> getImageTags() {
        return imageTags;
    }

    /**
     * Setter for property meta
     *
     * @param meta - string representation of metatags, associated to image. Used at jsf page.
     */
    public void setMeta(String meta) {
        this.meta = meta;
    }

    /**
     * Getter for property meta 
     * 
     * @return string representation of metatags, associated to image. Used at jsf page.
     */
    public String getMetaString(){
    	return meta;
    }
    
    /**
     * Getter for property cameraModel
     *
     * @return model of camera
     */
    public String getCameraModel() {
        return cameraModel;
    }

    /**
     * setter for property cameraModel
     *
     * @param cameraModel- model of camera
     */
    public void setCameraModel(String cameraModel) {
        this.cameraModel = cameraModel;
    }

    /**
     * Getter for property height
     *
     * @return height of image
     */
    public int getHeight() {
        return height;
    }

    /**
     * setter for property height
     *
     * @param height - height of image
     */
    public void setHeight(int height) {
        this.height = height;
    }

    /**
     * Getter for property size
     *
     * @return size of image in KB
     */
    public double getSize() {
        return size;
    }

    /**
     * setter for property size
     *
     * @param size - size of image in KB
     */
    public void setSize(double size) {
        this.size = size;
    }

    /**
     * Getter for property width
     *
     * @return width of image
     */
    public int getWidth() {
        return width;
    }

    /**
     * setter for property width
     *
     * @param width - width of image
     */
    public void setWidth(int width) {
        this.width = width;
    }

    /**
     * Getter for property uploaded
     *
     * @return date of upload to site of this image
     */
    public Date getUploaded() {
        return uploaded;
    }

    /**
     * setter for property uploaded
     *
     * @param uploaded - date of upload
     */
    public void setUploaded(Date uploaded) {
        this.uploaded = uploaded;
    }
    
    /**
     * Getter for property allowComments. If true, other user may comment this image.
     *
     * @return is other users may comment this image
     */
    public boolean isAllowComments() {
        return allowComments;
    }

    /**
     * @param allowComments the allowComments to set
     */
    public void setAllowComments(boolean allowComments) {
        this.allowComments = allowComments;
    }

    /**
     * 
     * @return if this image is covering for containing album
     */
    public boolean isCovering() {
        return this.equals(this.getAlbum().getCoveringImage());
    }

    /**
     * 
     * @param covering - determine if this image is covering for containing album
     */
    public void setCovering(boolean covering) {
        this.covering = covering;
    }
    
    /**
     * Getter for property visited
     *
     * @return boolean value, that indicated is user visit this image already
     */
    public boolean isVisited() {
		return visited;
	}

    /**
     * Setter for property visited
     *
     * @param visited - boolean value, that indicated is user visit this image already
     */
	public void setVisited(boolean visited) {
		this.visited = visited;
	}
	
	/**
     * Determine if this image should be marked as new(on jsf page, for example in tree)
     *
     * @return boolean value, that indicated is this image should be marked as new
     */
	public boolean isNew(){
		if(!visited){
			return this.getUploaded().after(ActionTools.getRecentlyDate());
		}
		return false;
	}
    
    //---------------------------Business methods

    /**
     * Add comment to this image.
     *
     * @param comment - comment to add
     */
    public void addComment(Comment comment) {
        if (comment == null) {
            throw new IllegalArgumentException("Null comment!");
        }
        comment.setImage(this);
        comments.add(comment);
    }

    /**
     * Remove comment from list of comments, belongs to that image.
     *
     * @param comment - comment to delete
     */
    public void removeComment(Comment comment) {
        if (comment == null) {
            throw new IllegalArgumentException("Null comment");
        }
        if(comment.getImage().equals(this)){
        	comment.setImage(null);
            comments.remove(comment);
        }else{
        	throw new IllegalArgumentException("Comment not belongs to this image");
        }
    }

    /**
     * Add metatag to this image.
     *
     * @param metatag - metatag to add
     */
    public void addMetaTag(MetaTag metatag) {
        if (metatag == null) {
            throw new IllegalArgumentException("Null metatag!");
        }
        if (!imageTags.contains(metatag)) {
            metatag.addImage(this);
            imageTags.add(metatag);
        }
    }

    /**
     * Remove metatag from list of metatag, associated to that image.
     *
     * @param metatag - metatag to delete
     */
    public void removeMetaTag(MetaTag metatag) {
        if (metatag == null) {
            throw new IllegalArgumentException("Null metatag!");
        }
        if (imageTags.contains(metatag)) {
            metatag.removeImage(this);
            imageTags.remove(metatag);
        }
    }

    /**
     * Return MetaTag object by string representation
     *
     * @param s - string representation of metatag
     */
    public MetaTag getTagByName(String s) {
        for (MetaTag t : imageTags) {
            if (t.getTag().equals(s)) {
                return t;
            }
        }
        return null;
    }

    /**
     * Return Comma separated tag value for presentation in view
     *
     */
    public String getMeta() {
        final StringBuilder s = new StringBuilder();
        for (MetaTag tag : this.imageTags) {
            s.append(tag.getTag()).append(", ");
        }
        //Remove ',' from end
		if (s.length() >= 2) {
			s.delete(s.length() - 2, s.length());
		}
		return s.toString();
    }
    
    /**
     * Return relative path of this image in file-system(relative to uploadRoot parameter)
     *
     */
    public String getFullPath(){
		if(getAlbum() != null && getAlbum().getOwner() != null && getAlbum().getPath() != null){
			return getAlbum().getPath() + this.path;
		}
		return null;
	}
    
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Image image = (Image) o;

        if (id != null ? !id.equals(image.id) : image.id != null) return false;
        if (path != null ? !path.equals(image.path) : image.path != null) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = (id != null ? id.hashCode() : 0);
        result = 31 * result + (path != null ? path.hashCode() : 0);
        return result;
    }
}
