/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.realworld.service;

import java.util.List;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;

import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.richfaces.realworld.domain.Album;
import org.richfaces.realworld.domain.Image;

/**
 * Class for manipulating with album entity. Analogous to DAO pattern.
 *  EJB3 Bean
 *
 * @author Andrey Markhel
 */
@Name("albumAction")
@Stateless
@AutoCreate
public class AlbumAction implements IAlbumAction {

	@In(value="entityManager")
	EntityManager em;

	/**
     * Persist album entity to database
     * @param album - album to add
     */
	public void addAlbum(Album album) {
		em.persist(album);
		//Add to shelf
		album.getShelf().addAlbum(album);
		em.flush();
	}
	
	/**
     * Remove album entity from database
     * @param album - album to delete
     */
	public void deleteAlbum(Album album){
		if(album.getShelf() == null){
			return;
		}
		//Remove from previous shelf 
		album.getShelf().removeAlbum(album);
		em.remove(album);
		em.flush();
	}
	
	/**
     * Synchronize state of album entity with database
     * @param album - album to Synchronize
     */
	public void editAlbum(Album album){
		em.flush();
	}

	/**
     * Used to retrieve all unvisited images, belongs to specified album.
     *
     * @param album
     * @return List of unvisited images
     */
	@SuppressWarnings("unchecked")
	public List<Image> getUnvisitedImages(Album album){
		return (List<Image>)em.createNamedQuery(Constants.ALBUM_UNVISITED_QUERY).setParameter(Constants.ALBUM_PARAMETER, album).setParameter(Constants.DATE_PARAMETER, ActionTools.getRecentlyDate()).getResultList();
	}
}
