/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.realworld.manager;

import java.io.Serializable;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.contexts.Contexts;
import org.jboss.seam.core.Events;
import org.richfaces.event.DataScrollerEvent;
import org.richfaces.realworld.domain.Album;
import org.richfaces.realworld.domain.Image;
import org.richfaces.realworld.domain.MetaTag;
import org.richfaces.realworld.domain.Shelf;
import org.richfaces.realworld.domain.User;
import org.richfaces.realworld.service.Constants;

@Name("controller")
@Scope(ScopeType.EVENT)
public class Controller implements Serializable{

	private static final long serialVersionUID = 5656562187249324512L;
	
	@In @Out Model model;

	@In(scope = ScopeType.SESSION) User user;

	public void selectShelfs(){
		model.resetModel(NavigationEnum.ALL_SHELFS, user, null, null, null, null);
	}
	
	public void selectAlbums(){
		model.resetModel(NavigationEnum.ALL_ALBUMS, user, null, null, null, null);
	}
	
	public void selectImages(){
		model.resetModel(NavigationEnum.ALL_IMAGES, user, null, null, null, user.getAllImages() );
	}
	
	@Restrict("#{s:hasRole('admin')}")
	public void startEditShelf(Shelf shelf){
		if(!canViewShelf(shelf)){
			pushEvent(Constants.ADD_ERROR_EVENT, Constants.HAVENT_ACCESS);
			return;
		}
		model.resetModel(NavigationEnum.SHELF_EDIT, shelf.getOwner(), shelf, null, null, null);		
	}
	
	public void cancelEditShelf(Shelf shelf){
		model.resetModel(NavigationEnum.SHELF_EDIT, shelf.getOwner(), shelf, null, null, null);		
	}

	public void showAlbum(Album album){
		if(!canViewAlbum(album)){
			pushEvent(Constants.ADD_ERROR_EVENT, Constants.HAVENT_ACCESS);
			return;
		}
		model.resetModel(NavigationEnum.ALBUM_PREVIEW, album.getOwner(), album.getShelf(), album, null, album.getImages());	
	}
	
	public void showImage(Image image){
		if(!canViewImage(image)){
			pushEvent(Constants.ADD_ERROR_EVENT, Constants.HAVENT_ACCESS);
			return;
		}
		model.resetModel(NavigationEnum.ALBUM_IMAGE_PREVIEW, image.getAlbum().getOwner(), image.getAlbum().getShelf(), image.getAlbum(), image, image.getAlbum().getImages());
		image.getAlbum().visitImage(image, true);
	}
	
	public void scroll(DataScrollerEvent e){
		Integer index = e.getPage();
		Image image = model.getSelectedAlbum().getImages().get(index - 1);
		model.resetModel(NavigationEnum.ALBUM_IMAGE_PREVIEW, image.getAlbum().getOwner(), image.getAlbum().getShelf(), image.getAlbum(), image, image.getAlbum().getImages());
		image.getAlbum().visitImage(image, true);
	}
	
	@Restrict("#{s:hasRole('admin')}")
	public void startEditImage(Image image){
		if(!canViewImage(image)){
			pushEvent(Constants.ADD_ERROR_EVENT, Constants.HAVENT_ACCESS);
			return;
		}
		model.resetModel(NavigationEnum.ALBUM_IMAGE_EDIT, image.getAlbum().getShelf().getOwner(), image.getAlbum().getShelf(), image.getAlbum(), image, image.getAlbum().getImages());		
	}
	
	@Restrict("#{s:hasRole('admin')}")
	public void editUser(){
		pushEvent(Constants.EDIT_USER_EVENT);
		model.resetModel(NavigationEnum.ALL_SHELFS, user, model.getSelectedShelf(), model.getSelectedAlbum(), model.getSelectedImage(), model.getImages());
	}
	
	public void cancelEditUser(){
		pushEvent(Constants.CANCEL_EDIT_USER_EVENT);
		model.resetModel(NavigationEnum.ALL_SHELFS, user, model.getSelectedShelf(), model.getSelectedAlbum(), model.getSelectedImage(), model.getImages());
	}
	
	public void cancelEditImage(Image image){
		model.resetModel(NavigationEnum.ALBUM_IMAGE_PREVIEW, image.getAlbum().getShelf().getOwner(), image.getAlbum().getShelf(), image.getAlbum(), image, image.getAlbum().getImages() );		
	}
	
	public void showShelf(Shelf shelf){
		model.resetModel(NavigationEnum.SHELF_PREVIEW, shelf.getOwner(), shelf, null, null, null);
	}
	
	@Restrict("#{s:hasRole('admin')}")
	public void startEditAlbum(Album album){
		if(!album.getOwner().getLogin().equals(user.getLogin())){
			pushEvent(Constants.ADD_ERROR_EVENT, Constants.HAVENT_ACCESS);
			return;
		}
		model.resetModel(NavigationEnum.ALBUM_EDIT, album.getShelf().getOwner(), album.getShelf(), album, null, album.getImages());
	}
	
	public void cancelEditAlbum(Album album){
		model.resetModel(NavigationEnum.ALBUM_PREVIEW, album.getShelf().getOwner(), album.getShelf(), album, null, album.getImages());
	}
	
	@Observer(Constants.ALBUM_ADDED_EVENT)
	public void onAlbumAdded(Album album){
		model.resetModel(NavigationEnum.ALBUM_PREVIEW, album.getShelf().getOwner(), album.getShelf(), album, null, album.getImages());
	}
	
	@Observer(Constants.ALBUM_EDITED_EVENT)
	public void onAlbumEdited(Album album){
		model.resetModel(NavigationEnum.ALBUM_PREVIEW, model.getSelectedUser(), model.getSelectedShelf(), album, null, album.getImages());
	}
	
	@Observer(Constants.ALBUM_DELETED_EVENT)
	public void onAlbumDeleted(Album album, String path){
		model.resetModel(NavigationEnum.ALL_ALBUMS, model.getSelectedUser(), model.getSelectedShelf(), null, null, null);
	}
	
	@Observer(Constants.SHELF_DELETED_EVENT)
	public void onShelfDeleted(Shelf shelf, String path){
		model.resetModel(NavigationEnum.ALL_SHELFS, model.getSelectedUser(), null, null, null, null);
	}
	
	@Observer(Constants.SHELF_ADDED_EVENT)
	public void onShelfAdded(Shelf shelf){
		model.resetModel(NavigationEnum.SHELF_PREVIEW, shelf.getOwner(), shelf, null, null, null);
	}
	
	@Observer(Constants.SHELF_EDITED_EVENT)
	public void onShelfEdited(Shelf shelf){
		model.resetModel(NavigationEnum.SHELF_PREVIEW, shelf.getOwner(), shelf, null, null, null);
	}
	
	@Observer(Constants.IMAGE_DELETED_EVENT)
	public void onImageDeleted(Image image, String path){
		model.resetModel(NavigationEnum.ALBUM_PREVIEW, image.getAlbum().getOwner(), image.getAlbum().getShelf(), image.getAlbum(), null, image.getAlbum().getImages());
	}
	
	@Observer(Constants.AUTHENTICATED_EVENT)
	public void onAuthenticate(User u){
		model.resetModel(NavigationEnum.ALL_SHELFS, u, null, null, null, null);
	}
	
	public void showFileUpload(){
		if(!(user.getShelfs().size() > 0)){
			pushEvent(Constants.ADD_ERROR_EVENT, Constants.FILE_UPLOAD_SHOW_ERROR);
			return;
		}
		model.resetModel(NavigationEnum.FILE_UPLOAD, model.getSelectedUser(), model.getSelectedShelf(), model.getSelectedAlbum(), model.getSelectedImage(), model.getImages());
	}	
	
	public void showFileUpload(Album album){
		model.resetModel(NavigationEnum.FILE_UPLOAD, album.getShelf().getOwner(), album.getShelf(), album, null, album.getImages());
	}
	
	public void showSharedAlbums(User user){
		UserManager userManager = (UserManager)Contexts.getEventContext().get("userManager");
		model.resetModel(NavigationEnum.USER_SHARED_ALBUMS, user, null, null, null, userManager.getSharedImages(model.getSelectedUser()));
	}
	
	public void showSharedImages(User user){
		UserManager userManager = (UserManager)Contexts.getEventContext().get("userManager");
		model.resetModel(NavigationEnum.USER_SHARED_IMAGES, user, null, null, null, userManager.getSharedImages(model.getSelectedUser()));
	}
	
	public void showUser(User user){
		model.resetModel(NavigationEnum.USER_PREFS, user, null, null, null, null);
	}
	
	public void showUnvisitedImages(Shelf shelf){
		ShelfManager shelfManager = (ShelfManager)Contexts.getEventContext().get("shelfManager");
		model.resetModel(NavigationEnum.SHELF_UNVISITED, shelf.getOwner(), shelf, null, null, shelfManager.getUnvisitedImages(shelf));
	}
	
	public void showUnvisitedImages(Album album){
		AlbumManager albumManager = (AlbumManager)Contexts.getEventContext().get("albumManager");
		model.resetModel(NavigationEnum.ALBUM_UNVISITED, album.getShelf().getOwner(), album.getShelf(), album, null, albumManager.getUnvisitedImages(album));
	}

	public void showTag(MetaTag metatag){
		model.resetModel(NavigationEnum.TAGS, model.getSelectedUser(), model.getSelectedShelf(), model.getSelectedAlbum(), model.getSelectedImage(), metatag.getImages());
		model.setSelectedTag(metatag);
	}
	
	public void showError(String error){
		pushEvent(Constants.ADD_ERROR_EVENT, error);
	}
	
	public boolean isUserImage(Image image){
		if(image == null || image.getAlbum() == null || image.getAlbum().getOwner() == null){
			return false;
		}
		return image.getAlbum().getOwner().getLogin().equals(user.getLogin());
	}
	
	public boolean isUserShelf(Shelf shelf){
		return shelf.getOwner()!=null && shelf.getOwner().getLogin().equals(user.getLogin());
	}
	
	public boolean isUserAlbum(Album album){
		if(album == null || album.getOwner() == null){
			return false;
		}
		return album.getOwner().getLogin().equals(user.getLogin());
	}
	
	private boolean canViewShelf(Shelf shelf) {
		return shelf.getOwner().getLogin().equals(user.getLogin());
	}
	
	private boolean canViewImage(Image image) {
		return image.getAlbum().getShelf().isShared() || image.getAlbum().getOwner().getLogin().equals(user.getLogin());
	}
	
	private boolean canViewAlbum(Album album) {
		return album.getShelf().isShared() || album.getOwner().getLogin().equals(user.getLogin());
	}
	
	private void pushEvent(String type, Object... parameters) {
		Events.instance().raiseEvent(type, parameters);
	}
	
	public boolean isProfileEditable(User selectedUser){
		if(selectedUser == null){
			return false;
		}
		if(selectedUser.getLogin().equals(user.getLogin())){
			return true;
		}
		return false;
	}
}