/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.realworld.manager;

import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.core.Events;
import org.richfaces.component.Dropzone;
import org.richfaces.event.DropEvent;
import org.richfaces.event.DropListener;
import org.richfaces.realworld.domain.Album;
import org.richfaces.realworld.domain.Image;
import org.richfaces.realworld.domain.Shelf;
import org.richfaces.realworld.domain.User;
import org.richfaces.realworld.service.Constants;
import org.richfaces.realworld.service.IAlbumAction;

@Name("dndManager")
public class DnDManager implements DropListener {

	@In User user;
	
	@In FileManager fileManager;
	
	@In IAlbumAction albumAction;
	
	@Restrict("#{s:hasRole('admin')}")
	public void processDrop(DropEvent dropEvent) {
		Dropzone dropzone = (Dropzone) dropEvent.getComponent();
		Object dragValue = dropEvent.getDragValue();
		Object dropValue = dropzone.getDropValue();
		if(dragValue instanceof Image){
				if(!((Album)dropValue).getOwner().getLogin().equals(user.getLogin())){
					Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, Constants.DND_PHOTO_ERROR);
					return;
				}
				handleImage((Image)dragValue, (Album)dropValue);	
		}else if(dragValue instanceof Album){
			if(!((Shelf)dropValue).getOwner().getLogin().equals(user.getLogin())){
				Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, Constants.DND_ALBUM_ERROR);
				return;
			}
			handleAlbum((Album)dragValue, (Shelf)dropValue);
		}
	}

	private void handleAlbum(Album dragValue, Shelf dropValue) {
		dropValue.addAlbum(dragValue);
		try{
			albumAction.editAlbum(dragValue);
		}catch(Exception e){
			Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, Constants.ERROR_IN_DB);
			return;
		}
		Events.instance().raiseEvent(Constants.CLEAR_TREE_EVENT);
	}

	private void handleImage(Image dragValue, Album dropValue) {
		if(dragValue.getAlbum().equals(dropValue)){
			return;
		}
		dropValue.addImage(dragValue);
		try{
			albumAction.editAlbum(dropValue);
		}catch(Exception e){
			Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, Constants.ERROR_IN_DB);
			return;
		}
		Events.instance().raiseEvent(Constants.DNDUPLOADED_EVENT, dragValue);
	}
}
