/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.realworld.manager;

import java.io.File;
import java.io.Serializable;
import java.util.List;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.core.Events;
import org.richfaces.realworld.domain.Album;
import org.richfaces.realworld.domain.Image;
import org.richfaces.realworld.domain.Shelf;
import org.richfaces.realworld.domain.User;
import org.richfaces.realworld.service.Constants;
import org.richfaces.realworld.service.IUserAction;

@Name("userManager")
@Scope(ScopeType.EVENT)
@AutoCreate
public class UserManager implements Serializable{

	private static final long serialVersionUID = 6027103521084558931L;
	
	@In(scope=ScopeType.SESSION) @Out(scope=ScopeType.SESSION) User user;
	
	@In FileManager fileManager;
	
	@In(required=false, scope=ScopeType.CONVERSATION) @Out(required=false, scope=ScopeType.CONVERSATION) File avatarData;
	
	@In IUserAction userAction;

	public Long countImages() {
		if(null == user.getCountImages() ){
			user.setCountImages(userAction.countImages());
		}
		return user.getCountImages();
	}
	
	public Long countSharedImages(User u) {
		return userAction.countSharedImages(u);
	}

	public Long countSharedAlbums(User u) {
		return userAction.countSharedAlbums(u);
	}
	
	public Long countAlbums() {
		if(null == user.getCountAlbums() ){
			user.setCountAlbums(userAction.countAlbums());
		}
		return user.getCountAlbums();
	}
	
	public List<Album> getSharedAlbums(User user){
		return userAction.getSharedAlbums(user);
	}
	
	public List<Image> getSharedImages(User user){
		return userAction.getSharedImages(user);
	}
	
	@Observer(Constants.EDIT_USER_EVENT)
	public void editUser(){
		if (avatarData != null) {
			if(!fileManager.saveAvatar(avatarData.getPath(), user)){
				Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, Constants.FILE_IO_ERROR);
				return;
			}
			avatarData.delete();
			avatarData = null;
			user.setHasAvatar(true);
		}try{
			user = userAction.updateUser();
		}catch(Exception e){
			Events.instance().raiseEvent(Constants.ADD_ERROR_EVENT, Constants.UPDATE_USER_ERROR);
			return;
		}
	}
	
	@Observer(Constants.CANCEL_EDIT_USER_EVENT)
	public void cancelEditUser() {
		avatarData = null;
	}
	
	@Observer(Constants.IMAGE_ADDED_EVENT)
	public void onImageAdded() {
		user.updateStatistics();
	}

	@Observer(Constants.IMAGE_DELETED_EVENT)
	public void onImageDeleted(Image image, String path){
		user.updateStatistics();
	}
	
	@Observer(Constants.ALBUM_DELETED_EVENT)
	public void onAlbumDeleted(Album album, String path){
		user.updateStatistics();
	}
	
	@Observer(Constants.ALBUM_ADDED_EVENT)
	public void onAlbumAdded(Album album){
		user.updateStatistics();
	}
	
	@Observer(Constants.SHELF_DELETED_EVENT)
	public void onShelfDeleted(Shelf shelf, String path){
		user.updateStatistics();
	}
}